# -*- coding: utf-8 -*-


# import utils_fit
import os

import fit_T_dep
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.constants
from matplotlib.backends.backend_agg import FigureCanvasAgg
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

KB = scipy.constants.Boltzmann
hbar = scipy.constants.hbar

# --- plotting parameters --- #

import scienceplots  # noqa

plt.close("all")
plt.style.use(["science"])

# --- setup figure --- #

# sample is mpaf13_9_19

mm = 1 / 25.4
fig_width = 80
ratio = (1 + np.sqrt(5)) / 2
fig_height = fig_width / ratio

plt.close("all")

px = 1 / plt.rcParams["figure.dpi"]
fig = plt.figure(figsize=(327.2067 * px * 2, 230 * px * 2), tight_layout=True)

# fig=plt.figure(figsize=(fig_width * mm, 2 * fig_height * mm),tight_layout=True)

# fig, ax = plt.subplots(
#     2,
#     1,
#     figsize=(fig_width * mm, 2 * fig_height * mm),
#     constrained_layout=True,
#     sharex=True,
# )

shape = (2, 2)

ax0M = plt.subplot2grid(shape, (0, 0))
ax0P = plt.subplot2grid(shape, (1, 0))

ax0M.set_xticklabels([])

# ax0M.sharex(ax0P)

ax1 = plt.subplot2grid(shape, (0, 1))
ax2 = plt.subplot2grid(shape, (1, 1))
ax1.set_xticklabels([])

# ax2.sharex(ax1)

verbose = False


""" Panel 1 fitted raw data"""
import mylab
from lmfit import Minimizer, Parameters

x, y = "Frequency", ["Data"]

ff31, YY31, info31 = mylab.fetch(
    h5file="Data/Dir_coup/Dir_Coup.hdf5",
    sample="Dir_Coup",
    run="S31",
    x=x,
    y=y,
    indexes=[0],
)
ff41, YY41, info31 = mylab.fetch(
    h5file="Data/Dir_coup/Dir_Coup.hdf5",
    sample="Dir_Coup",
    run="S41",
    x=x,
    y=y,
    indexes=[0],
)
f_dir_coup = ff31[0] * 1e9
Dir_coup_directivity = np.abs(YY41[0] / YY31[0])

sample = "MPAF13_9_19"
folder = r"Data/" + sample
runs = os.listdir(folder)
runs = runs[1:]

T_list = []
indexes = [0]

# fig, ax = plt.subplots(2, 1, sharex=True)
# fig2, ax2 = plt.subplots(2, 1, sharex=True, figsize=(5, 5))
# fig3, ax3 = plt.subplots(2, 1, sharex=True)
cmap = cm.get_cmap("winter", len(runs) + 2)
mks = 2  # marker_size
# kappa0 = []
# k0_stderr = []
# kappa_ext = []
# kex_stderr = []
# omega0 = []
# f_matrix = []
# run=0
tau = -3.605e-07

remove_idx = [1, 2, 3, 4, 6]
for j, run in enumerate(runs):
    # run=int(run)
    h5file = folder + "/" + run
    XX, YY, info = mylab.fetch(
        h5file=h5file, sample=sample, run=0, x=x, y=y, indexes=indexes
    )
    T = info["run_attr"]["T"]
    T_list.append(T)

    f = XX[0][(XX[0] != 4.4) & (XX[0] != 4.0)] * 1e9

    # f_fine=np.arange(4.55e9,4.65e9,15e3)
    f_fine = f
    # f_matrix.append(f_fine)
    w = 2 * np.pi * f
    w_fine = 2 * np.pi * f_fine

    Y = YY[0][(XX[0] != 4.4) & (XX[0] != 4.0) & (XX[0] != 3.6)] * np.exp(
        -1j * 2 * np.pi * f * tau
    )
    S11_pars = Parameters()
    S11_pars.add("w0", value=w[np.argmin(np.abs(Y))], min=0, vary=True)
    S11_pars.add("k", value=2 * np.pi * 30e6, min=0, vary=True)
    S11_pars.add("kex", value=2 * np.pi * 30e6, min=0, vary=True)
    S11_pars.add("A", value=np.abs(Y)[0], vary=True)
    S11_pars.add("B", value=0, vary=True)
    S11_pars.add("phi0", value=0, vary=True)
    S11_pars.add("q", value=2 * np.pi * 0.5e6, vary=True)
    S11_pars.add("tau", value=0, vary=False)
    S11_pars.add("background", value=1, vary=False)
    S11_pars.add("directivity", value=0.11, vary=False)

    """Fit S11"""
    S11_fitter = Minimizer(
        fit_T_dep.fit_S11,
        S11_pars,
        fcn_args=(w, np.concatenate((np.real(Y), np.imag(Y)))),
    )
    S11_result = S11_fitter.minimize()

    S11_pars["directivity"].value = np.interp(
        S11_result.params["w0"].value / 2 / np.pi, f_dir_coup, Dir_coup_directivity
    )
    S11_pars.add("w0", value=S11_result.params["w0"].value, min=0, vary=False)

    S11_fitter = Minimizer(
        fit_T_dep.fit_S11,
        S11_pars,
        fcn_args=(w, np.concatenate((np.real(Y), np.imag(Y)))),
    )
    S11_result = S11_fitter.minimize()
    phi0 = S11_result.params["phi0"].value
    kex = S11_result.params["kex"].value
    k = S11_result.params["k"].value
    w0 = S11_result.params["w0"].value
    q = S11_result.params["q"].value
    print(phi0)

    """plot to check"""

    data_mag = np.abs(Y)
    # data_phase = np.angle(Y * np.exp(-1j * 2 * np.pi * f * S11_pars["tau"].value))
    data_phase = np.angle(
        Y
        * np.exp(-1j * 2 * np.pi * f * S11_pars["tau"].value)
        / (1 + 2j * q / (k + 2j * (w - w0)))
    )
    data_phase = np.unwrap(data_phase - phi0)
    if np.mean(data_phase) > -np.pi / 2:
        data_phase = data_phase - np.pi
    elif np.mean(data_phase) < (-np.pi * 1.5):
        data_phase = data_phase + np.pi

    # ax0M.plot(f*1e-9, data_mag/5e-4, ".", c=cmap(j), label=T,markersize=mks)
    # ax0P.plot(f*1e-9, data_phase, ".", c=cmap(j),markersize=mks)

    fitted = fit_T_dep.fit_S11(S11_result.params, w_fine)
    S11_fit_pars = S11_result.params

    S11_fit_pars.add("directivity", value=0, vary=False)
    S11_fit_pars.add("background", value=0, vary=False)
    fitted_wo_bkg = fit_T_dep.fit_S11(S11_fit_pars, w_fine)

    fit_mag = np.abs(fitted)
    # fit_phase = np.angle(fitted)
    # fit_phase=np.unwrap(np.angle(fitted_wo_bkg))
    fit_phase = np.angle(fitted / (1 + 2j * q / (k + 2j * (w - w0))))
    fit_phase = np.unwrap(fit_phase - phi0)
    if np.mean(fit_phase) > -np.pi / 2:
        fit_phase = fit_phase - np.pi
    elif np.mean(fit_phase) < (-np.pi * 1.5):
        fit_phase = fit_phase + np.pi

    # ax0M.plot(f_fine, fit_mag, c=cmap(j))
    # ax0P.plot(f_fine, fit_phase, c=cmap(j))
    if j not in remove_idx:
        ax0M.plot(f * 1e-9, data_mag / 5e-4, ".", c=cmap(j), label=T, markersize=mks)
        ax0P.plot(f * 1e-9, data_phase, ".", c=cmap(j), markersize=mks)
        ax0M.plot(f_fine * 1e-9, fit_mag / 5e-4, c="black")
        ax0P.plot(f_fine * 1e-9, fit_phase, c="black")

"customize"
ax0M.set_ylabel("$|{S_{11}}|$ (a.u.)")
ax0M.set_yticks([0.25, 0.5, 0.75, 1.0])
ax0P.set_ylabel("$\\angle {S_{11}}$ (rad)")
ax0P.set_ylim(bottom=-2.25 * np.pi)
ax0P.set_yticks([0.0, -np.pi, -2 * np.pi])
ax0P.set_yticklabels(["0", "$-\\pi$", "$-2\\pi$"])
ax0P.set_xlabel("frequency (GHz)")


"""horizontal cb"""
axins1 = inset_axes(
    ax0M,
    width="100%",  # width: 50% of parent_bbox width
    height="10%",  # height: 5%
    loc="lower right",
    bbox_to_anchor=(0.5, 0.1, 0.5, 0.4),
    bbox_transform=ax0M.transAxes,
)

sm = plt.cm.ScalarMappable(
    cmap=cmap, norm=plt.Normalize(vmin=T_list[0], vmax=T_list[-1])
)
cb = fig.colorbar(sm, cax=axins1, orientation="horizontal")
cb.set_label(r"T (K)", labelpad=-30)


# """vertical cb"""

# axins1 = inset_axes(
#     ax0M,
#     width="10%",  # width: 50% of parent_bbox width
#     height="100%",  # height: 5%
#     loc="lower right",
#     bbox_to_anchor=(0.63,-0.4,0.4,0.75), bbox_transform=ax0M.transAxes
# )

# sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=T_list[0], vmax=T_list[-1]))
# cb=fig.colorbar(sm, cax=axins1)
# cb.set_label(r"T (K)",labelpad=-35,rotation=0,y=0.75)

""" Panel 2 fitted temp dep"""
# --- fitting methods --- #


def find_nearest(array, value):
    array = np.asarray(array)
    idx = np.argmin(np.abs(array - value))
    return idx


from lmfit import Minimizer, Parameters

T_arr = np.array(
    [1.9, 2.1, 2.3, 2.5, 2.75, 3.0, 3.25, 3.5, 3.75, 4.0, 4.5, 5.5, 6.0, 6.5]
)


omega0 = np.array(
    [
        2.77540415e10,
        2.77432909e10,
        2.77326310e10,
        2.77142104e10,
        2.76758227e10,
        2.76196184e10,
        2.75408911e10,
        2.74323625e10,
        2.72930581e10,
        2.71600727e10,
        2.67212682e10,
        2.54085435e10,
        2.44723755e10,
        2.33297370e10,
    ]
)

kappa0 = np.array(
    [
        8.64329301e06,
        9.12750451e06,
        1.04117298e07,
        1.24896070e07,
        1.47630359e07,
        1.80374733e07,
        2.12900056e07,
        2.73854439e07,
        3.03757125e07,
        3.40503258e07,
        4.45854503e07,
        4.37670408e07,
        6.93614881e07,
        1.00632555e08,
    ]
)

kappa_ext = np.array(
    [
        2.15139828e08,
        2.11396591e08,
        2.11539661e08,
        2.16728238e08,
        2.11519383e08,
        2.13091277e08,
        2.12637531e08,
        2.10880411e08,
        2.03932282e08,
        1.99648442e08,
        1.96730230e08,
        1.53050328e08,
        1.53576645e08,
        1.24737482e08,
    ]
)


data = np.concatenate((omega0, kappa0, kappa_ext))
T_arr_fine = np.linspace(T_arr[0], T_arr[-1], 20)

Rex_guess = 50
L0_guess = Rex_guess / kappa_ext[0]
C_guess = 1 / (omega0[0] ** 2 * L0_guess)

# --- fitting with Mattis-Bardeen --- #
pars = Parameters()

pars.add("Tc", value=9.45, min=0, vary=True)
pars.add("a", value=1.85, min=1, vary=True)
pars.add("w00", value=omega0[0], vary=True)


def w0_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    w00 = params["w00"]
    D0 = a * Tc * KB
    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))
    if data is None:
        return w0
    else:
        return data - w0


def w0_T_TTF(params, T, data=None):
    Tc = params["Tc"]
    w00 = params["w00"]
    w0 = w00 / np.sqrt(fit_T_dep.fL1(T, Tc))
    if data is None:
        return w0
    else:
        return data - w0


pars_w0 = pars
fitter = Minimizer(w0_T, pars_w0, fcn_args=(T_arr, omega0))
result = fitter.minimize()

fitter_TTF = Minimizer(w0_T_TTF, pars, fcn_args=(T_arr, omega0))
result_TTF = fitter_TTF.minimize()

fit_pars_w0 = result.params
fit_pars_w0_TTF = result_TTF.params


Tc = fit_pars_w0["Tc"]
a = fit_pars_w0["a"]
w00 = fit_pars_w0["w00"]
w_fit = w0_T(fit_pars_w0, T_arr_fine)
w_TTF = w0_T_TTF(fit_pars_w0_TTF, T_arr_fine)

if verbose:
    print(fit_pars_w0)

# --- plot panel a --- #

ax1.plot(T_arr, omega0 / 2 / np.pi * 1e-9, "o", ms=4, label="Data", color="C0")
ax1.plot(T_arr_fine, w_fit / 2 / np.pi * 1e-9, "-", label="MB", color="C1")
ax1.plot(T_arr_fine, w_TTF / 2 / np.pi * 1e-9, "--", label="TTF", color="C2")
ax1.legend(frameon=False, fontsize=8)
ax1.set_ylabel("$\omega_0 / 2 \pi$ (GHz)")

# --- insert schematic --- #

schem = plt.imread("schematic_circuit_series_lc_direct.png", format="png")
axin = ax1.inset_axes([0.01, 0.1, 0.4, 0.5], transform=ax1.transAxes)
axin.imshow(schem)
axin.axis("off")

# --- fit cavity loss rates --- #


def kex_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    L0 = params["L0"]
    D0 = a * Tc * KB
    L = L0 * fit_T_dep.fL(T, Tc, D0)
    kex = 50 / (L)
    if data is None:
        return kex
    else:
        return kex - data


pars_kex = Parameters()
pars_kex.add("a", value=a, min=1, vary=False)
pars_kex.add("Tc", value=Tc, min=1, vary=False)
pars_kex.add("L0", value=50 / kappa_ext[0], min=0, vary=True)

fitter_kex = Minimizer(kex_T, pars_kex, fcn_args=(T_arr, kappa_ext))
result_kex = fitter_kex.minimize()

fit_pars_kex = result_kex.params
L0 = fit_pars_kex["L0"]

kex_fit = kex_T(result_kex.params, T_arr_fine)

# --- plot panel b --- #

ax2.plot(
    T_arr,
    kappa_ext / 2 / np.pi * 1e-6,
    "^",
    ms=4,
    color="C0",
    label="$\kappa_{{\mathrm{{ext}}}}$",
)
ax2.plot(
    T_arr_fine,
    kex_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{\mathrm{{ext}}}}$ fit",
)

if verbose:
    print(fit_pars_kex)

# --- fit internal loss rates --- #

pars_k0 = Parameters()
pars_k0.add("a", value=a, min=0, vary=False)
pars_k0.add("Tc", value=Tc, min=0, vary=False)
pars_k0.add("L0", value=L0, min=0, vary=False)
pars_k0.add("w00", value=w00, min=0, vary=False)
pars_k0.add("Rd", value=514e3, min=1, vary=False)
pars_k0.add("beta", value=11, min=0, vary=False)
pars_k0.add("Rd0", value=23.56e6, min=0, vary=False)


def k0_T(params, T, data=None):
    a = params["a"]
    Tc = params["Tc"]
    L0 = params["L0"]
    # RL0=params['RL0']
    Rd = params["Rd"]
    Rd0 = params["Rd0"]
    w00 = params["w00"]
    beta = params["beta"]

    D0 = a * Tc * KB

    w0 = w00 / np.sqrt(fit_T_dep.fL(T, Tc, D0))
    # RL0=D0*L0*np.pi/hbar
    RL0 = D0 * L0 * np.pi / hbar
    # L0=hbar*RL0/D0
    RL = RL0 * fit_T_dep.fR(T, Tc, D0)
    L = L0 * fit_T_dep.fL(T, Tc, D0)
    ZL = 1 / (1 / (1j * w0 * L) + 1 / RL + 1 / (Rd * np.exp(beta / T)) + 1 / Rd0)

    k0 = np.real(ZL) / (np.imag(ZL) / w0)
    if data is None:
        return k0
    else:
        return k0 - data


fitter_k0 = Minimizer(k0_T, pars_k0, fcn_args=(T_arr, kappa0))
result_k0 = fitter_k0.minimize()

fit_pars_k0 = result_k0.params

k0_fit = k0_T(fit_pars_k0, T_arr_fine)

ax2.plot(
    T_arr,
    kappa0 / 2 / np.pi * 1e-6,
    "v",
    ms=4,
    color="C0",
    label="$\kappa_{{0}}$",
)

ax2.plot(
    T_arr_fine,
    k0_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{0}}$ fit",
    label="fit",
)

if verbose:
    print(fit_pars_k0)


# --- plot internal loss rates in the quasi particle limit --- #
fit_pars_k0_QP = result_k0.params
fit_pars_k0_QP["Rd"].value = np.inf
fit_pars_k0_QP["Rd0"].value = np.inf
k0_QP = k0_T(fit_pars_k0_QP, T_arr_fine)

ax2.plot(
    T_arr_fine,
    k0_QP / 2 / np.pi * 1e-6,
    "-",
    color="C1",
)

# --- annotate curves --- #
ax2.annotate(
    "$\kappa_{{\mathrm{{ext}}}}$",
    xy=(2.5, 60),
)

ax2.annotate(
    "$\kappa_{{0}}$",
    xy=(2.5, 1),
)

ax2.annotate(
    "$\kappa_{{0}}^{{\mathrm{{QP}}}}$",
    xy=(3.5, 0.2),
)


# --- customise --- #

ax2.set_yscale("log")
ax2.set_ylim(1e-1, 1.5e2)
ax2.set_ylabel("$\kappa/ 2 \pi$ (MHz)")
ax2.set_xlabel("Temperature (K)")

ax2.legend(frameon=False, fontsize=8, loc="lower right")

# --- annotate sublabels --- #
annotate_params = dict(xy=(-0.2, 0.9), xycoords="axes fraction", annotation_clip=False)
ax0M.annotate(r"\textbf{(a)}", **annotate_params)
ax0P.annotate(r"\textbf{(b)}", **annotate_params)

ax1.annotate(r"\textbf{(c)}", **annotate_params)
ax2.annotate(r"\textbf{(d)}", **annotate_params)

# --- save --- #
# fig.savefig("figure2_series_lc_direct.png", dpi=600)
# fig.savefig("figure2_series_lc_direct.pdf", dpi=600)
canvas = FigureCanvasAgg(fig)
canvas.print_figure("figure2_series_lc_direct.eps", dpi=600)
