# -*- coding: utf-8 -*-

# sample is mpaf13_12_21

import os

# import utils_fit
import fit_T_dep
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.constants

KB = scipy.constants.Boltzmann
hbar = scipy.constants.hbar

import mylab
import scienceplots  # noqa
from lmfit import Minimizer, Parameters
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

# --- plotting parameters --- #


plt.close("all")
plt.style.use(["science"])

# --- setup figure --- #

mm = 1 / 25.4
fig_width = 80
ratio = (1 + np.sqrt(5)) / 2
fig_height = fig_width / ratio

plt.close("all")

px = 1 / plt.rcParams["figure.dpi"]
fig = plt.figure(figsize=(327.2067 * px * 2, 230 * px * 2), tight_layout=True)

# fig=plt.figure(figsize=(fig_width * mm, 2 * fig_height * mm),tight_layout=True)

# fig, ax = plt.subplots(
#     2,
#     1,
#     figsize=(fig_width * mm, 2 * fig_height * mm),
#     constrained_layout=True,
#     sharex=True,
# )

shape = (2, 2)

ax0M = plt.subplot2grid(shape, (0, 0))
ax0P = plt.subplot2grid(shape, (1, 0))
ax0M.set_xticklabels([])

ax1 = plt.subplot2grid(shape, (0, 1))
ax2 = plt.subplot2grid(shape, (1, 1))
ax1.set_xticklabels([])

verbose = False
CHECK = False

# --- import and process data --- #

x, y = "Frequency", ["Data"]

ff31, YY31, info31 = mylab.fetch(
    h5file="Dir_Coup.hdf5", sample="Dir_Coup", run="S31", x=x, y=y, indexes=[0]
)
ff41, YY41, info31 = mylab.fetch(
    h5file="Dir_Coup.hdf5", sample="Dir_Coup", run="S41", x=x, y=y, indexes=[0]
)
f_dir_coup = ff31[0] * 1e9
Dir_coup_directivity = np.abs(YY41[0] / YY31[0])

sample = "MPAF13_12_21"
folder = os.path.join("temperature_data", sample)
runs_unsorted = os.listdir(folder)
# sort
runs = np.sort(runs_unsorted)
runs = runs[0:10]

# runs = np.arange(11)
# runs=np.array([5])
# cmap = mpl.colormaps.get_cmap("copper")(np.linspace(0, 1, len(runs)))
cmap = cm.get_cmap("winter", len(runs) + 2)
mks = 4
# runs=np.arange(5)+5
T_list = []
indexes = [0]

kappa0 = []
kappa_ext = []
omega0 = []
f_matrix = []
# run=0
tau = -3.6e-07

remove_idx = [1]
for j, run in enumerate(runs):
    h5file = os.path.join(folder, run)
    XX, YY, info = mylab.fetch(
        h5file=h5file, sample=sample, run=0, x=x, y=y, indexes=indexes
    )
    T = info["run_attr"]["T"]
    T_list.append(T)

    f = XX[0][(XX[0] != 4.4) & (XX[0] != 4.0)] * 1e9

    Y = YY[0][(XX[0] != 4.4) & (XX[0] != 4.0) & (XX[0] != 3.6)] * np.exp(
        -1j * 2 * np.pi * f * tau
    )
    # f_fine=np.arange(4.55e9,4.65e9,15e3)
    f_fine = f
    f_matrix.append(f_fine)
    w = 2 * np.pi * f
    w_fine = 2 * np.pi * f_fine

    S11_pars = Parameters()
    S11_pars.add("w0", value=w[np.argmin(np.abs(Y))], min=0, vary=True)
    S11_pars.add("k", value=2 * np.pi * 1e6, min=0, vary=True)
    S11_pars.add("kex", value=2 * np.pi * 1e6, min=0, vary=True)
    S11_pars.add("A", value=np.abs(Y)[0], vary=True)
    S11_pars.add("B", value=0, vary=True)
    S11_pars.add("phi0", value=0, vary=True)
    S11_pars.add("q", value=2 * np.pi * 0.5e6, vary=True)
    S11_pars.add("tau", value=0, vary=False)
    S11_pars.add("background", value=1, vary=False)
    S11_pars.add("directivity", value=0.0, vary=False)

    S11_fitter = Minimizer(
        fit_T_dep.fit_S11,
        S11_pars,
        fcn_args=(w, np.concatenate((np.real(Y), np.imag(Y)))),
    )
    S11_result = S11_fitter.minimize()

    S11_pars["directivity"].value = np.interp(
        S11_result.params["w0"].value / 2 / np.pi, f_dir_coup, Dir_coup_directivity
    )
    S11_pars.add("w0", value=S11_result.params["w0"].value, min=0, vary=False)

    S11_fitter = Minimizer(
        fit_T_dep.fit_S11,
        S11_pars,
        fcn_args=(w, np.concatenate((np.real(Y), np.imag(Y)))),
    )
    S11_result = S11_fitter.minimize()

    phi0 = S11_result.params["phi0"].value
    kex = S11_result.params["kex"].value
    k = S11_result.params["k"].value
    w0 = S11_result.params["w0"].value
    q = S11_result.params["q"].value

    kappa_ext.append(S11_result.params["kex"].value)
    kappa0.append(S11_result.params["k"].value - S11_result.params["kex"].value)
    omega0.append(S11_result.params["w0"].value)

    # --- plot to check --- #

    data_mag = np.abs(Y)
    # data_phase = np.angle(Y * np.exp(-1j * 2 * np.pi * f * S11_pars["tau"].value))

    data_phase = np.angle(
        Y
        * np.exp(-1j * 2 * np.pi * f * S11_pars["tau"].value)
        / (1 + 2j * q / (k + 2j * (w - w0)))
    )
    data_phase = np.unwrap(data_phase - phi0)
    if np.mean(data_phase) > np.pi / 2:
        data_phase = data_phase - np.pi
    # elif np.mean(data_phase)<(-np.pi*1.5):
    #     data_phase=data_phase+np.pi

    # ax0M.plot(f, data_mag, ".", c=cmap(j), label=T,markersize=mks)
    # ax0P.plot(f, data_phase, ".", c=cmap(j),markersize=mks)

    fitted = fit_T_dep.fit_S11(S11_result.params, w_fine)
    # S11_fit_pars = S11_result.params

    # S11_fit_pars.add("directivity", value=0, vary=False)
    # S11_fit_pars.add("background", value=0, vary=False)
    # fitted_wo_bkg = fit_T_dep.fit_S11(S11_fit_pars, w_fine)

    fit_mag = np.abs(fitted)
    # fit_phase = np.angle(fitted)
    fit_phase = np.angle(fitted / (1 + 2j * q / (k + 2j * (w - w0))))
    fit_phase = np.unwrap(fit_phase - phi0)
    if np.mean(fit_phase) > np.pi / 2:
        fit_phase = fit_phase - np.pi
    # elif np.mean(fit_phase)<(-np.pi):
    #     fit_phase=fit_phase+np.pi

    if j not in remove_idx:
        ax0M.plot(f * 1e-9, data_mag / 0.85e-4, ".", c=cmap(j), label=T, markersize=mks)
        ax0P.plot(f * 1e-9, data_phase, ".", c=cmap(j), markersize=mks)
        ax0M.plot(f_fine * 1e-9, fit_mag / 0.85e-4, c="black")
        ax0P.plot(f_fine * 1e-9, fit_phase, c="black")

    # ax_c[2].plot(f_fine, np.abs(fitted_wo_bkg), ".", c=cmap[j], label=T)
    # ax_c[3].plot(f_fine, np.angle(fitted_wo_bkg), ".", c=cmap[j])

"customize"
ax0M.set_ylabel("$|{S_{11}}|$ (a.u.)")
# ax0M.set_yticks([0.25,0.5,0.75,1.0])
ax0P.set_ylabel("$\\angle {S_{11}}$ (rad)")
# ax0P.set_ylim(bottom=-2.25*np.pi)
ax0P.set_yticks([0.0, -np.pi, -2 * np.pi])
ax0P.set_yticklabels(["0", "$-\\pi$", "$-2\\pi$"])
ax0P.set_xlabel("frequency (GHz)")


"""horizontal cb"""
axins1 = inset_axes(
    ax0P,
    width="100%",  # width: 50% of parent_bbox width
    height="10%",  # height: 5%
    loc="lower right",
    bbox_to_anchor=(0.1, 0.1, 0.5, 0.4),
    bbox_transform=ax0P.transAxes,
)

sm = plt.cm.ScalarMappable(
    cmap=cmap, norm=plt.Normalize(vmin=T_list[0], vmax=T_list[-1])
)
cb = fig.colorbar(sm, cax=axins1, orientation="horizontal")
cb.set_label(r"T (K)", labelpad=-30)


Tc_guess = 8.55
C_guess = 10.925e-15
# T_arr = np.array(T_list)
T_arr = np.array(T_list) * 0.99773292 - 0.059
# """fitted w0,kex,k0 vs T from data"""
kappa_ext = np.array(kappa_ext)
kappa0 = np.array(kappa0)
omega0 = np.array(omega0)

"""fit resonance frequency from data in the simplified wo(T) for low temperatures"""


# L0_guess = 1.1758974549636036e-07
L0_guess = 107.47e-9
Ls0_guess = 0.2e-9
Rd_guess = 82.4e3
Rd0_guess = 83e6


# pars = Parameters()
# pars.add("C", value=C_guess, min=1e-15, vary=False)
# pars.add("Tc", value=9.456, max=11, vary=True)
# pars.add("Tc2", value=10.797, max=11, vary=True)
# pars.add("Rd", value=71142093.8619153, min=1, vary=True)
# pars.add("L0", value=1.1751039585838896e-07, min=1e-12, vary=True)
# pars.add("Ls0", value=2.0560331215335736e-10, min=1e-12, vary=True)
# pars.add("RL0", value=182229.2114418202, min=1e4, vary=True)
# pars.add("nu", value=933570153055.9084, min=1e9, vary=True)
# pars.add("f0", value=omega0[0]/2/np.pi, min=0, vary=False)
# pars.add("f_span", value=100e6, min=0, vary=False)

"""Best fit so far"""
pars = Parameters()
pars.add("C", value=C_guess, min=1e-15, vary=False)
pars.add("Tc", value=9.534, max=11, vary=True)
pars.add("Tc2", value=10.213, max=11, vary=False)
pars.add("Rd", value=Rd_guess, min=1, vary=True)
pars.add("Rd0", value=Rd0_guess, min=1, vary=False)
pars.add("beta", value=15, min=0, vary=False)
pars.add("L0", value=1.0767157176960751e-07, min=1e-12, vary=True)
pars.add("Ls0", value=2.056032011310549e-10, min=1e-12, vary=False)
pars.add("a", value=1.85, min=0, vary=False)
pars.add("f0", value=omega0[0] / 2 / np.pi, min=0, vary=False)
pars.add("f_span", value=100e6, min=0, vary=False)

# 898822192778.0403

fitter = Minimizer(
    fit_T_dep.MB_Data_SeriesLC_Ls_v2,
    pars,
    fcn_args=(T_arr, np.concatenate((omega0, kappa0, kappa_ext))),
)

result = fitter.minimize()

fit_params = result.params
C = fit_params["C"].value
Tc = fit_params["Tc"].value
Rd = fit_params["Rd"].value
L0 = fit_params["L0"].value
Ls0 = fit_params["Ls0"].value
Rd0 = fit_params["Rd0"].value
beta = fit_params["beta"].value
a = fit_params["a"].value
# fit_params=pars
if verbose:
    print(fit_params)


T_arr_fine = np.linspace(np.min(T_arr), np.max(T_arr), 50)
# T_arr_fine = np.sort(T_arr)
w_fit, k0_fit, kex_fit = np.split(
    fit_T_dep.MB_Data_SeriesLC_Ls_v2(fit_params, T_arr_fine), 3
)


(line,) = ax1.plot(
    T_arr, omega0 / 2 / np.pi * 1e-9, "o", ms=4, alpha=1.0, color="C0", label="Data"
)
ax1.plot(T_arr_fine, w_fit / 2 / np.pi * 1e-9, "-", color="C1", label="MB")
# ax1.annotate(
#     f"$T_c$ = {Tc:.2f} K (MB)",
#     xy=(0.45, 0.2),
#     xycoords="axes fraction",
#     fontsize=8,
# )

ax1.legend(frameon=False, fontsize=8)
ax1.set_ylabel("$\omega_0 / 2 \pi$ (GHz)")


# --- insert schematic --- #

schem = plt.imread("schematic_circuit_series_lc_ls.png", format="png")
axin = ax1.inset_axes([0.01, 0.05, 0.4, 0.6], transform=ax1.transAxes)
axin.imshow(schem)
axin.axis("off")

# --- plot decay rates --- #

(line1,) = ax2.plot(
    T_arr,
    kappa_ext / 2 / np.pi * 1e-6,
    "^",
    ms=4,
    color="C0",
    label="$\kappa_{{\mathrm{{ext}}}}$",
)
ax2.plot(
    T_arr_fine,
    kex_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{\mathrm{{ext}}}}$ fit",
)
(line,) = ax2.plot(
    T_arr,
    kappa0 / 2 / np.pi * 1e-6,
    "v",
    ms=4,
    label="$\kappa_{{0}}$",
)
ax2.plot(
    T_arr_fine,
    k0_fit / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="$\kappa_{{0}}$ fit",
    label="fit",
)


fit_pars_k0_QP = result.params
# fit_pars_k0_QP=result_k.params
fit_pars_k0_QP["Rd"].value = np.inf
fit_pars_k0_QP["Rd0"].value = np.inf
w_fit_QP, k0_fit_QP, kex_fit_QP = np.split(
    fit_T_dep.MB_Data_SeriesLC_Ls_v2(fit_pars_k0_QP, T_arr_fine), 3
)
ax2.plot(
    T_arr_fine,
    k0_fit_QP / 2 / np.pi * 1e-6,
    "-",
    color="C1",
    # label="MB pred.",
)


# --- annotate curves --- #
ax2.annotate(
    "$\kappa_{{\mathrm{{ext}}}}$",
    # xy=(2.3, 0.65),
    # xytext=(1.6, 0.6),
    xy=(1.8, 2.0),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=0.0",
    # ),
)

ax2.annotate(
    "$\kappa_{{0}}$",
    # xy=(2.6, 0.4),
    # xytext=(1.8, 0.15),
    xy=(1.8, 0.12),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=0.1",
    # ),
)

ax2.annotate(
    "$\kappa_{{0}}^{{\mathrm{{QP}}}}$",
    # xy=(3.7, 0.4),
    # xytext=(3, 0.15),
    xy=(2.0, 1e-2),
    # xycoords="data",
    # arrowprops=dict(
    #     arrowstyle="->",
    #     facecolor="black",
    #     connectionstyle="arc3, rad=-0.1",
    # ),
)

# --- customise --- #


ax2.set_ylabel("$\kappa/2\pi$ (MHz)")
ax2.set_xlabel("Temperature (K)")

ax2.set_yscale("log")
ax2.set_ylim(0.5e-2, None)

ax2.legend(frameon=False, fontsize=8, loc="lower right")


# cmap2 = mpl.colormaps.get_cmap("cool")(np.linspace(0, 1, len(runs)))
cmap2 = cm.get_cmap("cool", len(runs) + 2)

D0 = a * Tc * KB
RL0 = D0 * L0 / hbar

# f_new=np.arange(4.45e9,4.65e9,15e3)

S11_pars_wo_bkg = S11_pars
S11_pars_wo_bkg.add("background", value=0, vary=False)
S11_pars_wo_bkg.add("q", value=0, vary=False)
S11_pars_wo_bkg.add("directivity", value=0, vary=False)

if CHECK:
    fig_c2, ax_c2 = plt.subplots(2, 1)

for i, T in enumerate(T_arr):
    # D0 = hbar / np.pi * nu * Tc
    L = L0 * fit_T_dep.fL(T, Tc, D0)
    Ls = Ls0 * fit_T_dep.fL(T, Tc, D0)
    RL = RL0 * fit_T_dep.fR(T, Tc, D0)
    RLs = RL0 * Ls0 / L0 * fit_T_dep.fR(T, Tc, D0)
    Rp = 1 / (1 / RL + 1 / Rd0 + 1 / (Rd * np.exp(beta / T)))
    f_new = np.arange(-20e6, 20e6, 15e3) + omega0[i] / 2 / np.pi

    fit_final = fit_T_dep.S11_series(2 * np.pi * f_new, C, L, Ls, Rp, RLs)
    S11_pars_wo_bkg.add("w0", value=omega0[i], vary=False)
    S11_pars_wo_bkg.add("k", value=kappa0[i] + kappa_ext[i], vary=False)
    S11_pars_wo_bkg.add("kex", value=kappa_ext[i], vary=False)

    if CHECK:
        ax_c2[0].set_ylim(0, 1.1)
        ax_c2[0].plot(f_new, np.abs(fit_final), c=cmap2(i))
        ax_c2[1].plot(f_new, np.angle(fit_final), c=cmap2(i))

        ax_c2[0].plot(
            f_new,
            np.abs(fit_T_dep.fit_S11(S11_pars_wo_bkg, 2 * np.pi * f_new)),
            c=cmap(i),
        )
        ax_c2[1].plot(
            f_new,
            np.angle(fit_T_dep.fit_S11(S11_pars_wo_bkg, 2 * np.pi * f_new)),
            c=cmap(i),
        )

# --- annotate sublabels --- #
annotate_params = dict(xy=(-0.22, 0.9), xycoords="axes fraction", annotation_clip=False)
ax1.annotate(r"\textbf{(a)}", **annotate_params)
ax2.annotate(r"\textbf{(b)}", **annotate_params)

# --- save --- #
fig.savefig("figure3_series_lc_ls.png", dpi=600)
fig.savefig("figure3_series_lc_ls.pdf", dpi=600)

# if CHECK:
#     fig_c.savefig("figure_check_one.png")
#     fig_c2.savefig("figure_check_two.png")
