# -*- coding: utf-8 -*-
"""
Created on Mon Jun  7 10:28:48 2021

@author:Ermes
"""

import copy

import h5py
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm
from scipy import signal

# %% Import stuff


"""
Sample should be defined with full name as in H5 file
Run can be either string or an integer index  (the function will check internallty)


to do:
    function 'search run' based on parameter

to add in the H5 files:
    units of dataset
    stepping variable
"""


def fetch(
    sample="self_loop_test",
    run=0,
    x="Frequency",
    y="Amplitude",
    indexes=0,
    h5file="QAFM_Data.hdf5",
    y_format=None,
):
    info = {
        "label": [],
        "x_label": "",
        "y_label": "",
        "title": "",
        "run_attr": {},
        "legend_title": "",
        "step_value": [],
    }
    XX = []
    if type(y) is list:
        if len(y) > 1:
            YY = [[] for x in range(len(y))]
            info["y_label"] = ["" for x in range(len(y))]
        else:
            y = y[0]
            YY = []
    else:
        YY = []

    with h5py.File(h5file, "r") as fh:
        if type(run) is int:
            run = list(fh[sample])[run]
        # print(f"run:{run} ")
        info["run"] = run
        info["title"] = run
        for key in list(fh[sample][run].attrs.keys()):
            # print(f"{key}:{fh[sample][run].attrs[key]}")
            # info['run_attr'].append(key+':'+ str(fh[sample][run].attrs[key]))
            info["run_attr"][key] = fh[sample][run].attrs[key]
        if "Stepping" in list(fh[sample][run].attrs.keys()):
            step = fh[sample][run].attrs["Stepping"]
            info["legend_title"] = step
        else:
            step = None
        index_list = list(fh[sample][run])
        try:
            index_list.remove("Source code")
        except:
            print("no source code")
        if indexes == "all":
            indexes = np.arange(len(index_list))

        for index in indexes:
            if str(index) in list(
                fh[sample][run]
            ):  # check if there are strange indexes
                dataset = fh[sample][run][str(index)]
                if step:
                    if str(step) in list(dataset.attrs.keys()):
                        # info['label'].append(step+'(i='+str(index)+')'+'= ' +str(dataset.attrs[str(step)]))
                        info["label"].append(
                            "(i="
                            + str(index)
                            + ")"
                            + "= "
                            # + str(np.around(dataset.attrs[str(step)], decimals=3))
                            + f"{dataset.attrs[str(step)]:.3e}"
                        )
                        info["step_value"].append(dataset.attrs[str(step)])
                    else:
                        info["label"].append("(i=" + str(index) + ")" + ":Unknown")
                        info["step_value"].append(np.nan)
                else:
                    info["label"].append("(i=" + str(index) + ")" + ":Unknown")

                if x in list(dataset):
                    X = np.array(dataset[x])
                    XX.append(X)
                else:
                    print("x_data name not found, dataset contains:")
                    print(list(dataset))
                    return ([np.array([])], [np.array([])], info)

                if type(y) is list:
                    for j, _y in enumerate(y):
                        if _y in list(dataset):
                            Y = np.array(dataset[_y])
                            if "Unit" in list(dataset[_y].attrs.keys()):
                                info["y_label"][j] = (
                                    _y + " [" + str(dataset[_y].attrs["Unit"]) + "]"
                                )

                            else:
                                info["y_label"][j] = _y
                            YY[j].append(Y)
                        else:
                            print("y_data name not found, dataset contains:")
                            print(list(dataset))
                            return ([np.array([])], [np.array([])], info)

                else:
                    if y in list(dataset):
                        Y = np.array(dataset[y])
                        if "Unit" in list(dataset[y].attrs.keys()):
                            info["y_label"] = (
                                y + " [" + str(dataset[y].attrs["Unit"]) + "]"
                            )
                        else:
                            info["y_label"] = y
                        YY.append(Y)
                    else:
                        print("x_data name not found, dataset contains:")
                        print(list(dataset))
                        return ([np.array([])], [np.array([])], info)

            else:
                print(f"Warning: Index {index} not in run {run}")
        if (XX) and ("Unit" in list(dataset[x].attrs.keys())):
            scale, unit = check_magn(np.array(XX))
            XX = np.array(XX) * scale
            info["x_label"] = x + " [" + unit + str(dataset[x].attrs["Unit"]) + "]"
        else:
            info["x_label"] = x

    return (np.array(XX), np.array(YY), info)


#    return(XX,YY,info)


def check_magn(x):
    X = np.max(np.max(x))
    if X >= 1e9:
        scale = 1e-9
        unit_mult = "G"
    elif X >= 1e6 and X < 1e9:
        scale = 1e-6
        unit_mult = "M"
    elif X >= 1e3 and X < 1e6:
        scale = 1e-3
        unit_mult = "k"
    else:
        scale = 1
        unit_mult = ""
    return (scale, unit_mult)


def set_format(Y, y_format, rescale=1):
    Y_form = copy.deepcopy(Y)
    try:
        rescale_shape = np.shape(rescale)[0]
    except:
        rescale_shape = 0

    if len(np.shape(Y)) >= 3:
        for i in range(np.shape(Y)[0]):
            for j in range(np.shape(Y)[1]):
                if rescale_shape == np.shape(Y)[1]:
                    temp = copy.deepcopy(Y[i][j]) / rescale[j]
                else:
                    try:
                        temp = copy.deepcopy(Y[i][j]) / rescale
                    except:
                        temp = copy.deepcopy(Y[i][j])
                        print("warning: rescaling of data failed")
                if y_format == "dB":
                    Y_form[i][j] = 20 * np.log10(np.abs(temp))
                elif y_format == "dBP":
                    Y_form[i][j] = 10 * np.log10(np.abs(temp))
                elif y_format == "rad":
                    Y_form[i][j] = signal.detrend(np.unwrap(np.angle(temp)))
    else:
        for i in range(np.shape(Y)[0]):
            if rescale_shape == (np.shape(Y)[0]):
                if rescale[i]:
                    try:
                        temp = copy.deepcopy(Y[i]) / rescale[i]
                    except:
                        temp = copy.deepcopy(Y[i])
                        print("warning: rescaling of data failed")
                else:
                    print("trying to divide by 0, rescaling by 1 instead")
                    temp = copy.deepcopy(Y[i])
            else:
                if (rescale == 0) or (0 in np.array(rescale)):
                    print("trying to divide by 0, rescaling by 1 instead")
                    temp = copy.deepcopy(Y[i])
                else:
                    try:
                        temp = copy.deepcopy(Y[i]) / rescale
                    except:
                        temp = copy.deepcopy(Y[i])
                        print("warning: rescaling of data failed")

            if y_format == "dB":
                Y_form[i] = 20 * np.log10(np.abs(temp))
            elif y_format == "dBP":
                Y_form[i] = 10 * np.log10(np.abs(temp))
            elif y_format == "rad":
                Y_form[i] = signal.detrend(np.unwrap(np.angle(temp)))

    return Y_form


def myplot(XX, YY, info, sharex=False, marker=None, fmt=".", cmap="coolwarm"):
    colmap = cm.get_cmap(cmap, np.shape(XX)[0] + 1)
    try:
        title = info["legend_title"]
    except:
        title = ""

    if len(np.shape(YY)) >= 3:
        ny = np.shape(YY)[0]
    else:
        ny = 1
    fig, ax = plt.subplots(ny, 1, sharex=sharex)

    if ny == 1:
        for i in range(np.shape(XX)[0]):
            try:
                label = info["label"][i]
            except:
                label = ""
            ax.plot(XX[i], YY[i], fmt, label=label, marker=marker, c=colmap(i))
        try:
            ax.set_xlabel(info["x_label"])
        except:
            pass
        try:
            ax.set_ylabel(info["y_label"])
        except:
            pass
        ax.legend(title=title)
        ax.set_title(info["title"])

    elif ny > 1:
        for j in range(ny):
            for i in range(np.shape(YY)[1]):
                try:
                    label = info["label"][i]
                except:
                    label = ""
                ax[j].plot(
                    XX[i], YY[j][i], fmt, label=label, marker=marker, c=colmap(i)
                )
            try:
                ax[j].set_ylabel(info["y_label"][j])
            except:
                pass
        if sharex:
            try:
                ax[j].set_xlabel(info["x_label"])
            except:
                pass
        else:
            for j in range(ny):
                try:
                    ax[j].set_xlabel(info["x_label"])
                except:
                    pass

        ax[0].legend(title=title, bbox_to_anchor=(1, 1.05))
        ax[0].set_title(info["title"])

    else:
        print("Something strange happened to your data")
        return 0

    return ax


# def mybarplot(XX,YY,info,sharex=False,bottom=None):
#    try:
#        title=info['legend_title']
#    except:
#        title=""
#
#    if len(np.shape(YY))==3:
#        ny=np.shape(YY)[0]
#    else:
#        ny=1
#    fig,ax=plt.subplots(ny,1,sharex=sharex)
#
#    if ny==1:
#        for i in range(np.shape(XX)[0]):
#            try:
#                label=info['label'][i]
#            except:
#                label=""
#            ax.stem(XX[i],YY[i],label=label,bottom=bottom,markerfmt=' ')
#        try:
#            ax.set_xlabel(info['x_label'])
#        except:
#            pass
#        try:
#            ax.set_ylabel(info['y_label'])
#        except:
#            pass
#        ax.legend(title=title)
#    elif ny>1:
#        for j in range (ny):
#            for i in range(np.shape(YY)[1]):
#                try:
#                    label=info['label'][i]
#                except:
#                    label=""
#                ax[j].stem(XX[i],YY[j][i],label=label,bottom=bottom,markerfmt=' ')
#            try:
#                ax[j].set_ylabel(info['y_label'][j])
#            except:
#                pass
#        try:
#            ax[j].set_xlabel(info['x_label'])
#        except:
#            pass
#
#        ax[0].legend(title=title,bbox_to_anchor=(1, 1.05))
#    else:
#        print("Something strange happened to your data")
#        return 0


#    return (ax)
def search(
    samples="all",
    runs="all",
    indexes=None,
    keys=None,
    attrs=None,
    h5file="QAFM_Data.hdf5",
):
    with h5py.File(h5file, "r") as fh:
        if samples == "all":
            samples = list(fh)

        elif type(samples) is not list:
            samples = [samples]
        for sample in samples:
            if runs == "all":
                _runs = list(fh[sample])
            elif type(runs) is not list:
                _runs = [runs]
            for run_index, run in enumerate(_runs):
                if type(run) is int:
                    run = list(fh[sample])[run]
                if keys:
                    if type(keys) is not list:
                        keys = [keys]
                    temp = True
                    for key in keys:
                        temp = temp and (key in list(fh[sample][run].attrs.keys()))
                    if temp:
                        print(f"sample={sample}, run= {run} ({run_index})")

                if attrs:
                    if type(attrs) is not list:
                        attrs = [attrs]
                    temp = True
                    for attr in attrs:
                        temp = temp and (attr in list(fh[sample][run].attrs.values()))
                    if temp:
                        print(f"sample={sample}, run= {run} ({run_index})")

                if indexes:
                    if indexes == "all":
                        _indexes = list(fh[sample][run])
                        try:
                            _indexes.remove("Source code")
                        except:
                            pass
                    elif (type(indexes) is not list) and (
                        type(indexes) is not np.ndarray
                    ):
                        _indexes = [indexes]

                    for index in _indexes:
                        if keys:
                            if type(keys) is not list:
                                keys = [keys]
                            temp = True
                            for key in keys:
                                temp = temp and (
                                    key
                                    in list(fh[sample][run][str(index)].attrs.keys())
                                )
                            if temp:
                                print(
                                    "sample={}, run= {} ({}), index={}".format(
                                        sample, run, run_index, index
                                    )
                                )

                        if attrs:
                            if type(attrs) is not list:
                                attrs = [attrs]
                            temp = True
                            for attr in attrs:
                                temp = temp and (
                                    attr
                                    in list(fh[sample][run][str(index)].attrs.values())
                                )
                            if temp:
                                print(
                                    "sample={}, run= {} ({}), index={}".format(
                                        sample, run, run_index, index
                                    )
                                )
    return 0
