# -*- coding: utf-8 -*-
import os

import h5py
import matplotlib.pyplot as plt
import numpy as np
import scienceplots  # noqa

# --- plotting parameters --- #


plt.style.use(["science"])

# --- setup figure --- #


mm = 1 / 25.4
fig_width = 80
ratio = (1 + np.sqrt(5)) / 2
fig_height = fig_width / ratio

plt.close("all")
fig, ax = plt.subplots(
    figsize=(fig_width * mm, fig_height * mm),
    constrained_layout=True,
)

# --- methods --- #


def load_data(file_name: str, sample_name: str, run_name: str):
    with h5py.File(file_name, "r") as fh:
        # handle
        idx_str = f"{sample_name}/{run_name}/{0}"

        # load data
        freq_arr = np.asarray(fh[idx_str]["Frequency"])
        data_arr = np.asarray(fh[idx_str]["Data"])

    return dict(data=data_arr, freq=freq_arr)


# --- load data --- #

cwd_path = os.getcwd()
file_name = "Dir_Coup.hdf5"
sample_name = "Dir_Coup"

load_pathname = os.path.join(cwd_path, file_name)

s21_dict = load_data(load_pathname, sample_name, "loopback")
s31_dict = load_data(load_pathname, sample_name, "S31")
s41_dict = load_data(load_pathname, sample_name, "S41")

# --- process --- #

baseline = 20 * np.log10(np.abs(s21_dict["data"]))
coupling = 20 * np.log10(np.abs(s31_dict["data"]))
isolation = 20 * np.log10(np.abs(s41_dict["data"]))

directivity = isolation - coupling

# --- plot --- #

ax.plot(
    1e-9 * s31_dict["freq"],
    coupling - baseline,
    label="$\\vert S_{{31}} \\vert$ (coupling)",
)

ax.plot(
    1e-9 * s41_dict["freq"],
    isolation - baseline,
    label="$\\vert S_{{41}} \\vert$ (isolation)",
)

# --- customise --- #
ax.set_ylabel("$\\vert S \\vert$-parameter (dB)")
ax.set_xlabel("Frequency (GHz)")
ax.legend(fontsize=8)

# --- save --- #
fig.savefig("figure6_directivity.eps", dpi=600)
